@@ -31,7 +31,6 @@ from fnmatch import fnmatch
 import codecs
 import ctypes.util
 import distutils.core
-import glob
 import os
 import platform
 import re
@@ -64,7 +63,8 @@ from defaultpaths import autostart, autostart_home
 from meta import (author, author_ascii, description, longdesc, domain, name, 
 				  py_maxversion, py_minversion, version, version_tuple, 
 				  wx_minversion, author_email, script2pywname)
-from util_os import relpath
+from util_os import getenvu, relpath, safe_glob
+from util_str import safe_str
 appname = name
 
 bits = platform.architecture()[0][:2]
@@ -72,6 +72,14 @@ pypath = os.path.abspath(__file__)
 pydir = os.path.dirname(pypath)
 basedir = os.path.dirname(pydir)
 
+if sys.platform in ("darwin", "win32"):
+	# Adjust PATH so ctypes.util.find_library can find SDL2 DLLs (if present)
+	pth = getenvu("PATH")
+	libpth = os.path.join(pydir, "lib")
+	if not pth.startswith(libpth + os.pathsep):
+		pth = libpth + os.pathsep + pth
+		os.environ["PATH"] = safe_str(pth)
+
 config = {"data": ["tests/*.icc"],
 		  "doc": ["LICENSE.txt",
 				  "README.html",
@@ -80,6 +88,7 @@ config = {"data": ["tests/*.icc"],
 				  "theme/*.png",
 				  "theme/*.css",
 				  "theme/*.js",
+				  "theme/*.svg",
 				  "theme/icons/favicon.ico",
 				  "theme/slimbox2/*.css",
 				  "theme/slimbox2/*.js"],
@@ -121,7 +130,11 @@ config = {"data": ["tests/*.icc"],
 								  "theme/icons/10x10/*.png",
 								  "theme/icons/16x16/*.png",
 								  "theme/icons/32x32/*.png",
+								  "theme/icons/48x48/*.png",
 								  "theme/icons/72x72/*.png",
+								  "theme/icons/128x128/*.png",
+								  "theme/icons/256x256/*.png",
+								  "theme/icons/512x512/*.png",
 								  "theme/jet_anim/*.png",
 								  "theme/patch_anim/*.png",
 								  "theme/splash_anim/*.png",
@@ -171,7 +184,7 @@ plist_dict = {"CFBundleDevelopmentRegion": "English",
 			  "CFBundleSignature": "????",
 			  "CFBundleVersion": ".".join(map(str, version_tuple)),
 			  "NSHumanReadableCopyright": u"© %s %s" % (strftime("%Y"), author),
-			  "LSMinimumSystemVersion": "10.5.0"}
+			  "LSMinimumSystemVersion": "10.6.0"}
 
 
 class Target:
@@ -295,14 +308,14 @@ def get_data(tgt_dir, key, pkgname=None, subkey=None, excludes=None):
 	for pth in files:
 		if not filter(lambda exclude: fnmatch(pth, exclude), excludes or []):
 			data.append((os.path.normpath(os.path.join(tgt_dir, os.path.dirname(pth))),
-						 glob.glob(os.path.join(src_dir, pth))))
+						 safe_glob(os.path.join(src_dir, pth))))
 	return data
 
 
 def get_scripts(excludes=None):
 	# It is required that each script has an accompanying .desktop file
 	scripts = []
-	desktopfiles = glob.glob(os.path.join(pydir, "..", "misc",
+	desktopfiles = safe_glob(os.path.join(pydir, "..", "misc",
 							 appname.lower() + "*.desktop"))
 	def sortbyname(a, b):
 		a, b = [os.path.splitext(v)[0] for v in (a, b)]
@@ -355,6 +368,7 @@ def setup():
 	use_setuptools = not use_distutils or "--use-setuptools" in \
 		sys.argv[1:] or (os.path.exists("use-setuptools") and 
 						 not "--use-distutils" in sys.argv[1:])
+	use_sdl = "--use-sdl" in sys.argv[1:]
 
 	sys.path.insert(1, os.path.join(os.path.dirname(pydir), "util"))
 
@@ -515,6 +529,10 @@ def setup():
 			if is_rpm_build:
 				doc = os.path.join(os.path.sep, "usr", doc)
 
+	# Use bundled CA file
+	if sdist or do_py2app:
+		config["package_data"][name].append("cacert.pem")
+
 	# on Mac OS X and Windows, we want data files in the package dir
 	# (package_data will be ignored when using py2exe)
 	package_data = {
@@ -524,10 +542,7 @@ def setup():
 	}
 	if sdist and sys.platform in ("darwin", "win32"):
 		package_data[name].extend(["theme/icons/22x22/*.png",
-								   "theme/icons/24x24/*.png",
-								   "theme/icons/48x48/*.png",
-								   "theme/icons/128x128/*.png",
-								   "theme/icons/256x256/*.png"])
+								   "theme/icons/24x24/*.png"])
 	if sys.platform == "win32" and not do_py2exe:
 		package_data[name].append("theme/icons/*.ico")
 	# Scripts
@@ -558,18 +573,33 @@ def setup():
 			# Add python and pythonw
 			data_files.extend([(os.path.join(data, "lib"), [sys.executable,
 				os.path.join(os.path.dirname(sys.executable), "pythonw.exe")])])
-			# OpenAL DLLs for pyglet
-			openal32 = ctypes.util.find_library("OpenAL32.dll")
-			wrap_oal = ctypes.util.find_library("wrap_oal.dll")
-			if openal32:
-				oal = [openal32]
-				if wrap_oal:
-					oal.append(wrap_oal)
+			if use_sdl:
+				# SDL DLLs for audio module
+				sdl2 = ctypes.util.find_library("SDL2")
+				sdl2_mixer = ctypes.util.find_library("SDL2_mixer")
+				if sdl2:
+					sdl2_libs = [sdl2]
+					if sdl2_mixer:
+						sdl2_libs.append(sdl2_mixer)
+						data_files.append((os.path.join(data, "lib"), sdl2_libs))
+						config["excludes"]["all"].append("pyglet")
+					else:
+						print "WARNING: SDL2_mixer not found!"
 				else:
-					print "WARNING: wrap_oal.dll not found!"
-				data_files.append((data, oal))
-			else:
-				print "WARNING: OpenAL32.dll not found!"
+					print "WARNING: SDL2 not found!"
+			if not "pyglet" in config["excludes"]["all"]:
+				# OpenAL DLLs for pyglet
+				openal32 = ctypes.util.find_library("OpenAL32.dll")
+				wrap_oal = ctypes.util.find_library("wrap_oal.dll")
+				if openal32:
+					oal = [openal32]
+					if wrap_oal:
+						oal.append(wrap_oal)
+					else:
+						print "WARNING: wrap_oal.dll not found!"
+					data_files.append((data, oal))
+				else:
+					print "WARNING: OpenAL32.dll not found!"
 		elif sys.platform != "darwin":
 			# Linux
 			data_files.append((os.path.join(os.path.dirname(data), "appdata"),
@@ -579,14 +609,14 @@ def setup():
 											"applications"), 
 							   [os.path.join(pydir, "..", "misc", name.lower() + 
 											 ".desktop")] +
-							   glob.glob(os.path.join(pydir, "..", "misc",
+							   safe_glob(os.path.join(pydir, "..", "misc",
 													  name.lower() + "-*.desktop"))))
 			data_files.append((autostart if os.geteuid() == 0 or prefix.startswith("/")
 							   else autostart_home, 
 							   [os.path.join(pydir, "..", "misc", 
 											 "z-%s-apply-profiles.desktop" % name.lower())]))
 			data_files.append((os.path.join(os.path.dirname(data), "man", "man1"), 
-							   glob.glob(os.path.join(pydir, "..", "man", "*.1"))))
+							   safe_glob(os.path.join(pydir, "..", "man", "*.1"))))
 			if not skip_instrument_conf_files:
 				# device configuration / permission stuff
 				if is_rpm_build:
@@ -605,7 +635,7 @@ def setup():
 				else:
 					devconf_files = []
 					if os.path.isdir("/etc/udev/rules.d"):
-						if glob.glob("/dev/bus/usb/*/*"):
+						if safe_glob("/dev/bus/usb/*/*"):
 							# USB and serial instruments using udev, where udev 
 							# already creates /dev/bus/usb/00X/00X devices
 							devconf_files.append(
@@ -630,22 +660,35 @@ def setup():
 							if os.path.isfile(fname):
 								data_files.extend([(entry[0], [fname])])
 		for dname in ("10x10", "16x16", "22x22", "24x24", "32x32", "48x48",
-					  "72x72", "128x128", "256x256"):
-			# Only the 10x10, 16x16, 32x32 and 72x72 icons are used exclusively
-			# by the app, the other sizes of the app icon are used for the 
-			# desktop entry under Linux
+					  "72x72", "128x128", "256x256", "512x512"):
+			# Get all the icons needed, depending on platform
+			# Only the icon sizes 10, 16, 32, 72, 256 and 512 include icons
+			# that are used exclusively for UI elements.
+			# These should be installed in an app-specific location, e.g.
+			# under Linux $XDG_DATA_DIRS/DisplayCAL/theme/icons/
+			# The app icon sizes 16, 32, 48 and 256 (128 under Mac OS X),
+			# which are used for taskbar icons and the like, as well as the
+			# other sizes can be installed in a generic location, e.g.
+			# under Linux $XDG_DATA_DIRS/icons/hicolor/<size>/apps/
+			# Generally, icon filenames starting with the lowercase app name
+			# should be installed in the generic location.
 			icons = []
 			desktopicons = []
 			if sys.platform == "darwin":
-				largest_icon_size = "128x128"
+				largest_iconbundle_icon_size = "128x128"
 			else:
-				largest_icon_size = "256x256"
-			for iconpath in glob.glob(os.path.join(pydir, "theme", "icons", 
+				largest_iconbundle_icon_size = "256x256"
+			for iconpath in safe_glob(os.path.join(pydir, "theme", "icons", 
 												   dname, "*.png")):
 				if not os.path.basename(iconpath).startswith(name.lower()) or (
 					sys.platform in ("darwin", "win32") and 
-					dname in ("16x16", "32x32", "48x48", largest_icon_size)):
-					# We need all the sizes we use in get_icon_bundle
+					dname in ("16x16", "32x32", "48x48",
+							  largest_iconbundle_icon_size)):
+					# In addition to UI element icons, we also need all the app
+					# icons we use in get_icon_bundle under macOS/Windows,
+					# otherwise they wouldn't be included (under Linux, these
+					# are included for installation to the system-wide icon
+					# theme location instead)
 					icons.append(iconpath)
 				elif sys.platform not in ("darwin", "win32"):
 					desktopicons.append(iconpath)
@@ -688,7 +731,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					lines.append(o)
 				print o.rstrip()
 			if len(lines):
-				os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.3'
+				os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.5'
 				sp.call(lines[-1], shell = True)  # fix the library
 	else:
 		macros = [("UNIX", None)]
@@ -733,7 +776,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 
 	attrs = {
 		"author": author_ascii,
-		"author_email": author_email,
+		"author_email": author_email.replace("@", "_at_"),
 		"classifiers": [
 			"Development Status :: 5 - Production/Stable",
 			"Environment :: MacOS X",
@@ -833,6 +876,8 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 				"plist": plist_dict
 			}
 		}
+		if use_sdl:
+			attrs["options"]["py2app"]["frameworks"] = ["SDL2", "SDL2_mixer"]
 		attrs["setup_requires"] = ["py2app"]
 
 	if do_py2exe:
@@ -1007,17 +1052,17 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 		if not paths:
 			# If the installed files have not been recorded, use some fallback 
 			# logic to find them
-			paths = glob.glob(os.path.join(cmd.install_scripts, name))
+			paths = safe_glob(os.path.join(cmd.install_scripts, name))
 			if sys.platform == "win32":
 				if setuptools:
-					paths += glob.glob(os.path.join(cmd.install_scripts, 
+					paths += safe_glob(os.path.join(cmd.install_scripts, 
 													name + ".exe"))
-					paths += glob.glob(os.path.join(cmd.install_scripts, 
+					paths += safe_glob(os.path.join(cmd.install_scripts, 
 													name + "-script.py"))
 				else:
-					paths += glob.glob(os.path.join(cmd.install_scripts, 
+					paths += safe_glob(os.path.join(cmd.install_scripts, 
 													name + ".cmd"))
-			paths += glob.glob(os.path.join(cmd.install_scripts, name + 
+			paths += safe_glob(os.path.join(cmd.install_scripts, name + 
 											"_postinstall.py"))
 			for attrname in [
 				"data", 
@@ -1031,12 +1076,12 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 				if not path in paths:
 					# Using sys.version in this way is consistent with 
 					# setuptools
-					paths += glob.glob(path) + glob.glob(path + 
+					paths += safe_glob(path) + safe_glob(path + 
 						("-%(version)s-py%(pyversion)s*.egg" % {
 							"version": version, 
 							"pyversion": sys.version[:3]
 						})
-					) + glob.glob(path + 
+					) + safe_glob(path + 
 						("-%(version)s-py%(pyversion)s*.egg-info" % {
 							"version": version, 
 							"pyversion": sys.version[:3]
@@ -1058,6 +1103,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					"README-fr.html",
 					"argyll_instruments.json",
 					"beep.wav",
+					"cacert.pem",
 					"camera_shutter.wav",
 					"ColorLookupTable.fx",
 					name.lower() + ".desktop",
@@ -1076,7 +1122,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 				]:
 					path = os.path.join(data, fname)
 					if not path in paths:
-						paths += glob.glob(path)
+						paths += safe_glob(path)
 			if os.path.isabs(doc) and not doc in paths:
 				for fname in [
 					"screenshots",
@@ -1087,7 +1133,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 				]:
 					path = os.path.join(doc, fname)
 					if not path in paths:
-						paths += glob.glob(path)
+						paths += safe_glob(path)
 			if sys.platform == "win32":
 				from postinstall import get_special_folder_path
 				startmenu_programs_common = get_special_folder_path(
@@ -1097,7 +1143,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					if path:
 						for filename in (name, "LICENSE", "README", 
 										 "Uninstall"):
-							paths += glob.glob(os.path.join(path, name, 
+							paths += safe_glob(os.path.join(path, name, 
 															filename + ".lnk"))
 
 		for path in paths:
@@ -1264,7 +1310,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 			if sys.platform == "win32":
 				path = os.path.join(cmd.install_lib, name)
 				# Using sys.version in this way is consistent with setuptools
-				for path in glob.glob(path) + glob.glob(
+				for path in safe_glob(path) + safe_glob(
 					os.path.join(path + (
 						"-%(version)s-py%(pyversion)s*.egg" % 
 						{
